@import url("http://www.thespacedoctor.co.uk/assets/css/main.css");

@import url("https://maxcdn.bootstrapcdn.com/font-awesome/4.5.0/css/font-awesome.min.css");

div.results > span.tiny {
    font-size: 6px;
}

ul.no-bullet {
     list-style-type: none;
     padding-left: 10px;
}


div.gollum-thumbnail-gallery {
    margin-right: 5px !important;
    margin-top: 10px !important;
}

img.gollum-thumbnail {
    height: 60px;
    float: left;
    text-align: center;
    border: 1px solid #ddd;
    border-radius: 4px;
    padding: 2px;
    margin-right: 5px !important;
    margin-bottom: 10px;
}
